#include "checkersui.h"
#include "ui_checkersui.h"
#include <QtGui>
#include <QtCore>
#include <QTime>
#include <QTimeEdit>
#include <QMessageBox>
#include "game.h"

CheckersUI::CheckersUI(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::CheckersUI)
{
    is_clicked = false;
    ui->setupUi(this);
    for(int i=0;i<8;i++){
        for(int j=0;j<8;j++){
                ui->Playground->setItem(i,j, new QTableWidgetItem(""));
        }
    }
    started=false;
    dif_player1=0;
    dif_player2=0;
    ui->theme2->setChecked(true);
    ui->actionDizzy->setChecked(true);
    ui->warning->setText("");
    cur_board = Board();
    set_warn(2);
    theme=1;//dizzy
    refresh_board();
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    timer->start(1000);

    ui->Playground->blockSignals(true);
}

CheckersUI::~CheckersUI()
{
    delete ui;
}


void CheckersUI::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void CheckersUI::refresh_board(){
    int temp;
    if(theme==0){
        for(int i=0;i<8;i++){
            for(int j=0;j<8;j++){
                 temp=cur_board.piece(i,j);
                 if((i+j)%2==1) {
                    switch (temp) {
                     case -2:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/wooden/Rking.bmp"));
                        break;
                    case -1:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/wooden/Rcheck.bmp"));
                        break;
                    case 0:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/wooden/Bsquare.bmp"));
                        break;
                    case 1:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/wooden/Wcheck.bmp"));
                        break;
                    case 2:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/wooden/Wking.bmp"));
                        break;
                    }
                 }else{
                  ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/wooden/Wsquare.bmp"));
                }
            }
        }
    }else if(theme==1){
        for(int i=0;i<8;i++){
            for(int j=0;j<8;j++){
                 temp=cur_board.piece(i,j);
                 if((i+j)%2==1) {
                    switch (temp) {
                     case -2:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/dizzy/Rking.jpg"));
                        break;
                    case -1:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/dizzy/Rcheck.jpg"));
                        break;
                    case 0:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/dizzy/Bsquare.jpg"));
                        break;
                    case 1:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/dizzy/Wcheck.jpg"));
                        break;
                    case 2:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/dizzy/Wking.jpg"));
                        break;
                    }
                 }else{
                  ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/dizzy/Wsquare.jpg"));
                }
             }
        }
    }else if(theme==2){
        for(int i=0;i<8;i++){
            for(int j=0;j<8;j++){
                 temp=cur_board.piece(i,j);
                 if((i+j)%2==1) {
                    switch (temp) {
                     case -2:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/mario/bowser.jpg"));
                        break;
                    case -1:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/mario/wario.jpg"));
                        break;
                    case 0:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/mario/Bsquare.jpg"));
                        break;
                    case 1:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/mario/yoshi.jpg"));
                        break;
                    case 2:
                        ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/mario/mario.jpg"));
                        break;
                    }
                 }else{
                  ui->Playground->item(i,j)->setBackground(QImage("../checkers/debug/mario/Wsquare.jpg"));
                }
             }
        }
    }
    ui->Playground->repaint();
    return;
}

void CheckersUI::set_warn(int num_error) {
    if(ui->warning->text()=="Game over!"){
        return;
    }
    QString er;
    switch (num_error) {
    case 0:
        er = QString("Thinking ... Please wait!");
        break;
    case 1:
        if(theme==1){
            er = QString("Black Player:");
        }else if(theme==0){
            er = QString("Red Player:");
        }else{
            er = QString("Bad Team:");
        }

        break;
    case 2:
        if(theme==2){
            er = QString("Good Team:");
        }else{
            er = QString("White Player:");
        }
        break;
    case 3:
        er = QString("Continue your attack:");
        break;
    case 4:
        er = QString("Invalid move!");
        break;
    case 5:
        er = QString("Game over!");
        break;
    case 6:
        er = QString("Your second home please! ");
        break;
    case 7:
        er = QString("Set the AI level from the edit menu, please!");
        break;
    case 8:
        er = QString("You may click now!");
        break;
    }
    ui->warning->setText(er);
    ui->warning->repaint();

}

void CheckersUI::on_Playground_cellClicked(int row, int column)
{

    if(is_clicked==false){
      is_clicked = true;
      clickedx = row;
      clickedy = column;
      set_warn(6);
      return;
    }
    is_clicked = false;
    Move cur_move;
    cur_move.set(clickedx, clickedy, row, column);
    bool my_turn = (cur_board.turn_player()==-1);
    int ans = player[my_turn].input_move(clickedx, clickedy, row, column, cur_board);
    if (ans==-1) {
        set_warn(4);
        return;
    }
    if (ans==1) {
        refresh_board();
        set_warn(3);
        return;
    }
    cur_board.change_turn();
    refresh_board();
    if (player[!my_turn].is_computer_player()) {
        set_warn(0);
        ui->Playground->blockSignals(true);
        if (!player[!my_turn].play(cur_board)) {
            set_warn(5);
            return;
        }
    }
    refresh_board();
    ui->Playground->blockSignals(false);
    if (player[my_turn].is_finished(cur_board)) {
        set_warn(5);
        return;
    }
    if (cur_board.turn_player()==1){
        set_warn(2);
    }else{
        set_warn(1);
    }
}

void CheckersUI::start_game() {
    started=true;
    cur_board = Board();
    ui->warning->setText("");
    if(ui->theme1->isChecked()==1){
        theme=0;
    }else if (ui->theme2->isChecked()==1){
        theme=1;
    }else{
        theme=2;
    }
    is_clicked = false;
    player[0]._attack_= false;
    player[1]._attack_= false;
    player[0].set(ui->first_player->isChecked());
    player[1].set(ui->second_player->isChecked());
    if(dif_player1==0 && player[0].is_computer_player()==1){
        set_warn(7);
        return;
    }
    if(dif_player2==0 && player[1].is_computer_player()==1){
        set_warn(7);
        return;
    }
    player[0].set_difficulty(dif_player1);
    player[1].set_difficulty(dif_player2);
    refresh_board();

    QTime res(0,0,0,0);
    ui->timer->setTime(res);
    ui->timer2->setTime(res);
    bool _turn=false;
    bool is_finished=false;
    int counter=500;
    ui->Playground->blockSignals(true);
    for (;player[_turn].is_computer_player();_turn=!_turn,counter--) {
        if (player[_turn].is_computer_player()) {
            set_warn(0);
            if (player[_turn].play(cur_board)) {
                refresh_board();
            } else {
                refresh_board();
                set_warn(5);
                is_finished=true;
                break;
            }
        }
        if(counter==0){
            is_finished=true;
            set_warn(5);
            break;
        }
    }
    if(is_finished==false){
        set_warn(((int)(!_turn))+1);
        ui->Playground->blockSignals(false);
    }
    return;
}

void CheckersUI::update(){
    if(started==false){
        return;
    }
    if(cur_board.turn_player()==1){
        if(player[0].is_computer_player()==1){
            return;
        }else{
            QTime adder(ui->timer->time());
            adder = adder.addSecs(1);
            ui->timer->setTime(adder);
        }
    }else{
        if(player[1].is_computer_player()==1){
            return;
        }else{
            QTime adder(ui->timer2->time());
            adder = adder.addSecs(1);
            ui->timer2->setTime(adder);
        }
    }
}

void CheckersUI::on_actionNew_game_triggered()
{
    start_game();
}

void CheckersUI::on_start_game_clicked(bool checked)
{
    start_game();
}

void CheckersUI::on_actionExit_triggered()
{
    exit(0);
}

void CheckersUI::on_actionAbout_triggered()
{
    QMessageBox About;
    About.about(this,"RECheckers","RECheckers Written by: <br> Rahmtin Rotabi<br> Ehsan Emamjomezadeh");
}


void CheckersUI::on_actionWooden_triggered()
{
        ui->actionDizzy->setChecked(false);
        ui->theme2->setChecked(false);
        ui->theme1->setChecked(true);
        ui->actionWooden->setChecked(true);
        ui->theme3->setChecked(false);
        ui->actionMario->setChecked(false);
}

void CheckersUI::on_actionDizzy_triggered()
{
        ui->actionDizzy->setChecked(true);
        ui->theme2->setChecked(true);
        ui->theme1->setChecked(false);
        ui->actionWooden->setChecked(false);
        ui->theme3->setChecked(false);
        ui->actionMario->setChecked(false);
}

void CheckersUI::on_actionMario_triggered()
{
    ui->actionDizzy->setChecked(false);
    ui->theme2->setChecked(false);
    ui->theme1->setChecked(false);
    ui->actionWooden->setChecked(false);
    ui->theme3->setChecked(true);
    ui->actionMario->setChecked(true);
}


void CheckersUI::on_theme1_clicked()
{
    ui->actionDizzy->setChecked(false);
    ui->theme2->setChecked(false);
    ui->theme1->setChecked(true);
    ui->actionWooden->setChecked(true);
    ui->theme3->setChecked(false);
    ui->actionMario->setChecked(false);
}


void CheckersUI::on_theme2_clicked()
{
    ui->actionDizzy->setChecked(true);
    ui->theme2->setChecked(true);
    ui->theme1->setChecked(false);
    ui->actionWooden->setChecked(false);
    ui->theme3->setChecked(false);
    ui->actionMario->setChecked(false);
}

void CheckersUI::on_theme3_clicked()
{
    ui->actionDizzy->setChecked(false);
    ui->theme2->setChecked(false);
    ui->theme1->setChecked(false);
    ui->actionWooden->setChecked(false);
    ui->theme3->setChecked(true);
    ui->actionMario->setChecked(true);
}

void CheckersUI::on_actionEasy_triggered()
{

    ui->actionEasy->setChecked(true);
    ui->actionMedium->setChecked(false);
    ui->actionHard->setChecked(false);
    ui->actionExpert->setChecked(false);
    dif_player1=1;
}


void CheckersUI::on_actionMedium_triggered()
{

    ui->actionEasy->setChecked(false);
    ui->actionMedium->setChecked(true);
    ui->actionHard->setChecked(false);
    ui->actionExpert->setChecked(false);
    dif_player1=2;
}

void CheckersUI::on_actionHard_triggered()
{

    ui->actionEasy->setChecked(false);
    ui->actionMedium->setChecked(false);
    ui->actionHard->setChecked(true);
    ui->actionExpert->setChecked(false);
    dif_player1=3;
}

void CheckersUI::on_actionExpert_triggered()
{

    ui->actionEasy->setChecked(false);
    ui->actionMedium->setChecked(false);
    ui->actionHard->setChecked(false);
    ui->actionExpert->setChecked(true);
    dif_player1=4;
}

void CheckersUI::on_actionEasy_2_triggered()
{

    ui->actionEasy_2->setChecked(true);
    ui->actionMedium_2->setChecked(false);
    ui->actionHard_2->setChecked(false);
    ui->actionExpert_2->setChecked(false);
    dif_player2=1;
}

void CheckersUI::on_actionMedium_2_triggered()
{

    ui->actionEasy_2->setChecked(false);
    ui->actionMedium_2->setChecked(true);
    ui->actionHard_2->setChecked(false);
    ui->actionExpert_2->setChecked(false);
    dif_player2=2;
}

void CheckersUI::on_actionHard_2_triggered()
{

    ui->actionEasy_2->setChecked(false);
    ui->actionMedium_2->setChecked(false);
    ui->actionHard_2->setChecked(true);
    ui->actionExpert_2->setChecked(false);
    dif_player2=3;
}

void CheckersUI::on_actionExpert_2_triggered()
{

    ui->actionEasy_2->setChecked(false);
    ui->actionMedium_2->setChecked(false);
    ui->actionHard_2->setChecked(false);
    ui->actionExpert_2->setChecked(true);
    dif_player2=4;
}


