#include "player.h"
#include <cstdlib>
#include <ctime>

Player::Player() {
	_attack_ = false;
	is_computer = false;
	MAX_LEVEL = 6;
}

void Player::set(bool _is_computer) {
	is_computer = _is_computer;
}

void Player::set_difficulty(int d) { // 0 < d < 5
	MAX_LEVEL = (2*d);
	return;
}

bool Player::is_computer_player() {
	return is_computer;
}

int Player::find_max_level(Board & cur_board) {
	return MAX_LEVEL;
}

int Player::my_inf(Board & cur_board) {
	return (cur_board.turn_player()*cur_board.inf());
}

bool Player::can_attack_piece(Board & cur_board, int row, int col) {
	int dir[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	Move temp;
	for (int i=0;i<4;i++) {
		temp.set(row, col, row+2*dir[i][0], col+2*dir[i][1]);
		if (temp.is_valid(cur_board))
			return true;
	}
	return false;
}

bool Player::can_attack(Board & cur_board) {
	for (int row=0;row<8;row++)
		for (int col=(1-(row%2));col<8;col+=2)
			if (can_attack_piece(cur_board, row, col))
				return true;
	return false;
}

bool Player::is_finished(Board & cur_board) {
	int dir[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	Move temp;
	for (int row=0;row<8;row++)
		for (int col=(1-(row%2));col<8;col+=2)
			for (int i=0;i<4;i++)
				for (int len=1;len<3;len++) {
					temp.set(row, col, row+len*dir[i][0], col+len*dir[i][1]);
					if (temp.is_valid(cur_board))
						return false;
				}
	return true;
}

int Player::find_value_attack_piece(Board & cur_board, int row, int col, int level, int alpha, int beta) {
	bool my_turn = ((bool)((1-cur_board.turn_player())/2)), find_attack = false;
	int best_value = (-my_inf(cur_board)), cur_value = 0;
	int alpha_beta[2] = {alpha, beta}, dir[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	Move temp;
	for (int i=0;i<4;i++) {
		temp.set(row, col, row+2*dir[i][0], col+2*dir[i][1]);
		if (temp.is_valid(cur_board)) {
			find_attack = true;
			temp.do_move(cur_board);
			// cur_value = find_value_attack_piece(cur_board, row+2*dir[i][0], col+2*dir[i][1], level, alpha_beta[0], alpha_beta[1]);
			/**/
			if (temp._changed_to_king()) {
				cur_board.change_turn();
				cur_value = find_value(cur_board, level-1, alpha, beta);
				cur_board.change_turn();
			} else {
				cur_value = find_value_attack_piece(cur_board, row+2*dir[i][0], col+2*dir[i][1], level, alpha_beta[0], alpha_beta[1]);
			}
			/**/
			temp.undo_move(cur_board);
			if (((cur_value-best_value)*cur_board.turn_player())>0) {
				best_value = cur_value;
				if ((best_value<alpha_beta[my_turn])==my_turn)
					alpha_beta[my_turn] = best_value;
				if (alpha_beta[0]>alpha_beta[1])
					break;
			}
		}
	}
	if (!find_attack) {
		cur_board.change_turn();
		best_value = find_value(cur_board, level-1, alpha, beta);
		cur_board.change_turn();
	}
	return best_value;
}

int Player::find_value_attack(Board & cur_board, int level, int alpha, int beta) {
	bool my_turn = ((bool)((1-cur_board.turn_player())/2));
	int best_value = (-my_inf(cur_board)), cur_value = 0;
	int alpha_beta[2] = {alpha, beta};
	for (int row=0;row<8;row++)
		for (int col=(1-(row%2));col<8;col+=2)
			if (can_attack_piece(cur_board, row, col)) {
				cur_value = find_value_attack_piece(cur_board, row, col, level, alpha_beta[0], alpha_beta[1]);
				if (((cur_value-best_value)*cur_board.turn_player())>0) {
					best_value = cur_value;
					if ((best_value<alpha_beta[my_turn])==my_turn)
						alpha_beta[my_turn] = best_value;
					if (alpha_beta[0]>alpha_beta[1])
						break;
				}
			}
	return best_value;
}

int Player::find_value_without_attack(Board & cur_board, int level, int alpha, int beta) {
	bool my_turn = ((bool)(1-cur_board.turn_player()));
	int best_value = (-my_inf(cur_board)), cur_value = 0;
	int alpha_beta[2] = {alpha, beta}, dir[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	Move temp;
	for (int row=0;row<8;row++)
		for (int col=(1-(row%2));col<8;col+=2)
			for (int i=0;i<4;i++) {
				temp.set(row, col, row+dir[i][0], col+dir[i][1]);
				if (temp.is_valid(cur_board)) {
					temp.do_move(cur_board);
					cur_board.change_turn();
					cur_value = find_value(cur_board, level-1, alpha_beta[0], alpha_beta[1]);
					cur_board.change_turn();
					temp.undo_move(cur_board);
					if (((cur_value-best_value)*cur_board.turn_player())>0) {
						best_value = cur_value;
						if ((best_value<alpha_beta[my_turn])==my_turn)
							alpha_beta[my_turn] = best_value;
						if (alpha_beta[0]>alpha_beta[1])
							break;
					}
				}
			}
	return best_value;
}

int Player::find_value(Board & cur_board, int level, int alpha, int beta) {
	if (level==0) {
		if (is_finished(cur_board))
			return (-cur_board.inf()*cur_board.turn_player());
		return cur_board.value();
	}
	if (can_attack(cur_board))
		return find_value_attack(cur_board, level, alpha, beta);
	return find_value_without_attack(cur_board, level, alpha, beta);
}

int Player::find_best_attack_piece(Board & cur_board, int row, int col, int level, int alpha, int beta, vector<Move> & best_move) {
	bool my_turn = ((bool)((1-cur_board.turn_player())/2)), find_attack = false, _temp_king = false;
	int best_value = (-my_inf(cur_board)), cur_value = 0;
	int alpha_beta[2] = {alpha, beta}, dir[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	Move temp;
	vector<Move> temp_v;
	for (int i=0;i<4;i++) {
		temp.set(row, col, row+2*dir[i][0], col+2*dir[i][1]);
		if (temp.is_valid(cur_board)) {
			find_attack = true;
			temp.do_move(cur_board);
			// cur_value = find_best_attack_piece(cur_board, row+2*dir[i][0], col+2*dir[i][1], level, alpha_beta[0], alpha_beta[1], temp_v);
			/**/
			if (temp._changed_to_king()) {
                                _temp_king = true;
				cur_board.change_turn();
				cur_value = find_value(cur_board, level-1, alpha, beta);
				cur_board.change_turn();
			} else {
				_temp_king = false;
				cur_value = find_best_attack_piece(cur_board, row+2*dir[i][0], col+2*dir[i][1], level, alpha_beta[0], alpha_beta[1], temp_v);
			}
			/**/
			temp.undo_move(cur_board);
			if (((cur_value-best_value)*cur_board.turn_player())>0) {
				best_value = cur_value;
                                if (_temp_king)
					best_move.clear();
				else
					best_move = temp_v;
				best_move.push_back(temp);
				if ((best_value<alpha_beta[my_turn])==my_turn)
					alpha_beta[my_turn] = best_value;
				if (alpha_beta[0]>alpha_beta[1])
					break;
			}
		}
	}
	if (!find_attack) {
		cur_board.change_turn();
		best_value = find_value(cur_board, level-1, alpha, beta);
		cur_board.change_turn();
		best_move.clear();
	}
	return best_value;
}

bool Player::find_best_attack(Board & cur_board, int level) {
	bool my_turn = ((bool)((1-cur_board.turn_player())/2));
	int best_value = (-my_inf(cur_board)), cur_value = 0;
	int alpha_beta[2] = {-cur_board.inf(), cur_board.inf()};
	vector<Move> temp, best_move;
	for (int row=0;row<8;row++)
		for (int col=(1-(row%2));col<8;col+=2)
			if (can_attack_piece(cur_board, row, col)) {
				cur_value = find_best_attack_piece(cur_board, row, col, level, alpha_beta[0], alpha_beta[1], temp);
				if (((cur_value-best_value)*cur_board.turn_player())>0) {
					best_value = cur_value;
					best_move = temp;
					if ((best_value<alpha_beta[my_turn])==my_turn)
						alpha_beta[my_turn] = best_value;
					if (alpha_beta[0]>alpha_beta[1])
						break;
				}
			}
	for (int i=(best_move.size()-1);i>=0;i--)
		best_move[i].do_move_player(cur_board);
	cur_board.change_turn();
	return true;
}

bool Player::find_best_without_attack(Board & cur_board, int level) {
	srand(time(0));
	bool my_turn = ((bool)(1-cur_board.turn_player())), find_ans = false;
	int best_value = (-my_inf(cur_board)), cur_value = 0;
	int alpha_beta[2] = {-cur_board.inf(), cur_board.inf()}, dir[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	Move temp, best_move;
	for (int row=0;row<8;row++)
		for (int col=(1-(row%2));col<8;col+=2)
			for (int i=0;i<4;i++) {
				temp.set(row, col, row+dir[i][0], col+dir[i][1]);
				if (temp.is_valid(cur_board)) {
					temp.do_move(cur_board);
					cur_board.change_turn();
					cur_value = find_value(cur_board, level-1, alpha_beta[0], alpha_beta[1]);
					cur_board.change_turn();
					temp.undo_move(cur_board);
					//using rand :
					if ((cur_value*cur_board.turn_player())>420) {
						cur_value += ((rand()%(2*cur_board.eps()+1))-cur_board.eps());
						if ((cur_board.piece(row, col)*cur_board.turn_player())==1)
							cur_value += (cur_board.eps()*cur_board.turn_player());
					}
					else {
						int temp_eps = ((cur_board.eps()/2)-1);
						cur_value += ((rand()%(2*temp_eps+1))-temp_eps);
						if ((cur_board.piece(row, col)*cur_board.turn_player())==1)
							cur_value += ((rand()%temp_eps)*cur_board.turn_player());
					}
					//
					if ((!find_ans)||(((cur_value-best_value)*cur_board.turn_player())>0)) {
						find_ans = true;
						best_value = cur_value;
						best_move = temp;
						if ((best_value<alpha_beta[my_turn])==my_turn)
							alpha_beta[my_turn] = best_value;
						if (alpha_beta[0]>alpha_beta[1])
							break;
					}
				}
			}
	if (find_ans) {
		best_move.do_move_player(cur_board);
		cur_board.change_turn();
	}
	return find_ans;
}

int Player::input_move(int first_0, int first_1, int second_0, int second_1 , Board & cur_board) {
	Move temp;
	temp.set(first_0, first_1, second_0, second_1);
	if (!temp.is_valid_input(cur_board))
		return -1;
	if (_attack_) {
		if (!temp.continue_attack(last_move))
			return -1;
		temp.do_move_player(cur_board);
		last_move = temp;
		if (!can_attack_piece(cur_board, second_0, second_1)) {
			_attack_ = false;
			return 0;
		}
		if (temp._changed_to_king()) {
			_attack_ = false;
			return 0;
		}
		return 1;
	}
	if (can_attack(cur_board)&&(temp.len()==1))
		return -1;
	if ((!can_attack(cur_board))&&(temp.len()==2))
		return -1;
	temp.do_move_player(cur_board);
	last_move = temp;
	if ((temp.len()==2)&&(can_attack_piece(cur_board, second_0, second_1))) {
		if (!temp._changed_to_king()) {
			_attack_ = true;
			return 1;
		}
	}
	_attack_ = false;
	return 0;
}

bool Player::play(Board & cur_board) {
	if (can_attack(cur_board))
		return find_best_attack(cur_board, find_max_level(cur_board));
	return find_best_without_attack(cur_board, find_max_level(cur_board));
}

